import os
import requests

from cc_py_commons.utils.logger_v2 import logger

GEOCODE_API_URL = os.environ.get('GEOCODE_API_URL')
GEOCODE_API_TOKEN = os.environ.get('GEOCODE_API_TOKEN')


def execute(city, state, zipcode):
	url = f'{GEOCODE_API_URL}?city={city}&state={state}'
	if zipcode:
		url = f'{url}&zip={zipcode}'
	token = f"Bearer {GEOCODE_API_TOKEN}"
	headers = {
		"Authorization": token
	}
	logger.debug(f"Geocoding location: URL: {url}, Headers: {headers}")

	response = requests.get(url, headers=headers)

	if response.status_code != 200:
		logger.warning(f"Request to geocode location failed with status code: {response.status_code}")
		return None

	return response.json()



import requests

from cc_py_commons.utils.logger_v2 import logger
from .constants import C4_API_AUTH_TOKEN, C4_API_URL


def execute(account_id, request_id=None):
  url = f"{C4_API_URL}/users/search"
  token = f"Bearer {C4_API_AUTH_TOKEN}"
  headers = {
    "Authorization": token
  }
  http_params = {
    "accountId": account_id,
    "requestId": request_id
  }

  logger.debug(f"Getting users for account: URL: {url}, Params: {http_params}")
  response = requests.get(url, params=http_params, headers=headers)

  if response.status_code != 200:
    logger.warning(f"Request to get user failed with status code: {response.status_code}")
    return None

  return response.json()



import requests

from cc_py_commons.config.env import app_config
from cc_py_commons.utils.logger_v2 import logger

def execute(contact_id, payload):
	uri = app_config.CARRIER_HUB_URL + '/carrier/-/contact/' + contact_id
	headers = {
		'Authorization': 'Bearer ' + app_config.CARRIER_HUB_AUTH_TOKEN,
		'Content-Type': 'application/json'
	}
	logger.debug(f"Updating contact {contact_id} in carrier-hub: {uri} with payload: {payload}")
	response = requests.post(uri, headers=headers, json=payload)
	if response.status_code == 200:
		logger.debug(f"Successfully updated the contact: {contact_id}")
		return response.json()
	else:
		logger.warning(f'Failed to update contact {response.status_code}:{response.text}')
	return None



import json

from cc_py_commons.config.env import app_config
from cc_py_commons.sns.sns_service import SnsService
from cc_py_commons.utils import json_logger


def send(c4_user_id, c4_account_id, event_type, payload):
	user_data = {'id': c4_user_id, 'accountId': c4_account_id}
	analytics_payload = {**payload, **user_data, 'eventType': event_type}
	json_logger.debug(c4_account_id, 'Sending analytics SNS notification',
					  topic=app_config.ANALYTICS_SNS_ARN, event_type=event_type, payload=analytics_payload)
	sns_service = SnsService()
	sns_service.send(app_config.ANALYTICS_SNS_ARN, analytics_payload.get('subject'), json.dumps(analytics_payload))



import os, sys

from urllib.parse import urlparse, uses_netloc
from redis import Redis
from cc_py_commons.config.env import app_config

if not app_config.LOCATION_REDIS_URL:
  raise RuntimeError('Set up Location Redis first.')

if not app_config.DISTANCE_CACHE_URL:
  raise RuntimeError('Set up Distance Cache first.')

if not app_config.TIMEZONE_CACHE_URL:
  raise RuntimeError('Set up Timezone Redis first.')

uses_netloc.append('redis')
_location_redis_url = urlparse(app_config.LOCATION_REDIS_URL)
_distance_cache_url = urlparse(app_config.DISTANCE_CACHE_URL)
_timezone_redis_url = urlparse(app_config.TIMEZONE_CACHE_URL)

location_db_conn = Redis(host=_location_redis_url.hostname, port=_location_redis_url.port, db=0, password=app_config.LOCATION_REDIS_PWD)

distance_db_conn = Redis(host=_distance_cache_url.hostname, port=_distance_cache_url.port, password=app_config.DISTANCE_CACHE_PASSWORD)

timezone_db_conn = Redis(host=_timezone_redis_url.hostname, port=_timezone_redis_url.port, db=0, password=app_config.TIMEZONE_CACHE_PASSWORD)



import pandas as pd
import s3fs
import boto3

from cc_py_commons.config.env import app_config
from cc_py_commons.utils.logger_v2 import logger

# Initializes s3fs to use AWS KEY/SECRET and returns URL for file
s3 = s3fs.S3FileSystem(key=app_config.ACCESS_KEY_ID, secret=app_config.SECRET_ACCESS_KEY)


def does_exist(bucket, filename):
  s3_url = get_s3_url(bucket, filename)
  return s3.exists(s3_url)

def load_data(file_name, bucket_name):
  data = pd.DataFrame()

  if not does_exist(bucket_name, file_name): 
    logger.warning(f"s3_file.load_data: Trying to load {bucket_name} and file: {file_name} but could not locate it.")
    return data

  s3_url = get_s3_url(bucket_name, file_name)
  logger.debug(f"s3_file.load_data: loading from {s3_url}")

  if (".csv" in file_name.lower()):
    data = pd.read_csv(s3_url, skipinitialspace=True)
    data.rename(columns=lambda x: x.strip().lower(), inplace=True)
  else:
    data = pd.read_excel(s3_url)

  return data

def load_data_as_string(file_name, bucket_name, skip_rows=0):
  data = pd.DataFrame()

  if not does_exist(bucket_name, file_name):
    logger.warning(f"S3 File: Trying to load {bucket_name} and file: {file_name} but could not locate it.")
    return data

  s3_url = get_s3_url(bucket_name, file_name)
  logger.debug(f"load_data_as_string: loading from {s3_url}")

  if (".csv" in file_name.lower()):
    data = pd.read_csv(s3_url, skipinitialspace=True)
    data.rename(columns=lambda x: x.strip().lower(), inplace=True)
  else:
    data = pd.read_excel(s3_url, skiprows=skip_rows, dtype=str)

  return data

def load_data_as_string_without_headers(file_name, bucket_name, skip_rows=0):
  data = pd.DataFrame()

  if not does_exist(bucket_name, file_name):
    logger.warning(f"S3 File: Trying to load {bucket_name} and file: {file_name} but could not locate it.")
    return data

  s3_url = get_s3_url(bucket_name, file_name)
  logger.debug(f"load_data_as_string_without_headers: loading from {s3_url}")

  if (".csv" in file_name.lower()):
    data = pd.read_csv(s3_url, skipinitialspace=True, skiprows=skip_rows, header=None)
    data.rename(columns=lambda x: x.strip().lower(), inplace=True)
  else:
    data = pd.read_excel(s3_url, skiprows=skip_rows, dtype=str, header=None)

  return data

def get_s3_url(bucket_name, filename):
  """Returns URL for file"""
  return f"s3://{bucket_name}/{filename}"

def write_file(bucket_name, filename, file_data, content_type=None):
  client = boto3.client('s3')
  client.put_object(Bucket=bucket_name, Key=filename, Body=file_data, ContentType=content_type)



def get_table_name(bulk_search_id):
	"""
	SHOULD NOT BE USED GOING FORWARD. Restoring for mercury dependency until all code is released.
	"""
	bulk_search_id_string = str(bulk_search_id).replace('-', '')
	return f'bulk_lane_pricing_results_{bulk_search_id_string}'



import json

from cc_py_commons.sns.sns_service import SnsService

class ImportMovementsNotification:

  def __init__(self, app_config):
    self._app_config = app_config

  def send(self, mcleod_token, c4_account_id, company_id):
    message = '{' + f'"accountId" : {c4_account_id}, ' \
                    f'"mcleodToken" : "{mcleod_token}", ' \
                    f'"companyId": "{company_id}", ' \
                    f'"subject" : "{self._app_config.IMPORT_MOVEMENTS_SNS_SUBJECT}", ' \
                    f'"className":  "{self._app_config.IMPORT_MOVEMENTS_SNS_CLASS_NAME}"' + '}'
    sns_service = SnsService()
    sns_service.send(self._app_config.IMPORT_MOVEMENTS_SNS_TOPIC_ARN,
      self._app_config.IMPORT_MOVEMENTS_SNS_SUBJECT, message)



import uuid
from dataclasses import dataclass, field
@dataclass
class Contact:
  email_address: str
  id: uuid.UUID = field(default=None)
  carrier_id: uuid.UUID = field(default=None)
  first_name: str = field(default=None)
  last_name: str = field(default=None)
  phone: str = field(default=None)
  mobile: str = field(default=None)
  primary_contact: bool = field(default=False)
  is_mail_reciever: bool = field(default=False)



from cc_py_commons.db import connection as db_connection


def execute(table_name):
	conn = None
	try:
		conn = db_connection.get()
		cursor = conn.cursor()
		sql_query = f"CREATE TABLE {table_name} (position smallint, search_result text, error text)"
		cursor.execute(sql_query)
		conn.commit()
	finally:
		db_connection.close(conn)
	return table_name



from uuid import UUID, uuid4
import datetime
import unittest

from cc_py_commons.loads.location import Location
from cc_py_commons.loads.equipment import Equipment
from cc_py_commons.loads.load import Load
from cc_py_commons.transactions.map_transaction import from_load

class TestMapTransaction(unittest.TestCase):

	def setUp(self):

		self.valid_load = Load(reference_number='20067', 
			customer_id=5876, 
			origin=Location(city='PLYMOUTH', state='NC', postcode='27962', county=None, country=None, latitude=None, longitude=None), 
			destination=Location(city='COLLEGE PARK', state='GA', postcode='30349', county=None, country=None, latitude=None, longitude=None), 
			pickup_date=datetime.date(2021, 7, 23), 
			delivery_date=datetime.date(2021, 7, 23), 
			status_id=UUID('9bcd4613-831e-4cb4-a76f-9166ce559fa6'), 
			source_id=UUID('03caea36-98c9-46f6-aa9c-a102576bc185'), 
			equipment=[Equipment(id='836284b0-f1ae-4f78-83e6-93f81b4c22c9', name='REEFER')], 
			equipment_description='REEFER', 
			contact=None, 
			pickup_open_time=datetime.time(0, 1), 
			pickup_close_time=None, 
			pickup_appointment_required=True, 
			delivery_open_time=datetime.time(0, 1), 
			delivery_close_time=None, 
			delivery_appointment_required=True, 
			tracking_number=None, 
			full_load=True, 
			length=None, 
			width=None, 
			height=None, 
			weight=35078.0,
			load_count=None, 
			distance=534.0, 
			stops=2, 
			rate=138786, 
			declared_value=None, 
			comment=None, 
			commodity='Nuts', 
			min_temperature=60.0, 
			max_temperature=None, 
			tarp_size=None, 
			carrier_id=None, 
			contact_id=None, 
			url=None, 
			demo_load=False, 
			team_service_required=False, 
			quote_id=None, 
			truck_lane_search_id=None, 
			truck_search_id=None, 
			id=None,
			load_number=None, 
			mcleod_movement_id='20067',
			request_id=uuid4())

	def test_valid_load(self):
		t = from_load(self.valid_load)
		self.assertEqual(self.valid_load.reference_number, t.reference_number)
		self.assertEqual(self.valid_load.request_id, t.request_id)
		self.assertEqual(self.valid_load.customer_id, t.user_id)



import os

import requests
from dateutil.parser import parse

from cc_py_commons.bids.bid_schema import BidSchema
from cc_py_commons.utils import json_logger

BOOKING_AGENT_URL = os.environ.get('BOOKING_AGENT_URL')
BOOKING_AGENT_TOKEN = os.environ.get("BOOKING_AGENT_TOKEN")


def execute(account_id, filters_dict):
	url = f"{BOOKING_AGENT_URL}/bid/match"
	token = f"Bearer {BOOKING_AGENT_TOKEN}"
	headers = {
		"Authorization": token
	}
	json_logger.debug(account_id, f"Requesting bids from booking-agent", url=url, filters=filters_dict)
	response = requests.get(url, headers=headers, params=filters_dict)
	bids = []
	if response.status_code == 200:
		bids = response.json()['content']
		for bid in bids:
			bid['pickupDate'] = parse(bid['pickupDate']).strftime('%Y-%m-%d')
			bid['deliveryDate'] = parse(bid['deliveryDate']).strftime('%Y-%m-%d')
			if bid.get('inviteEmailedAt'):
				bid['inviteEmailedAt'] = parse(bid['inviteEmailedAt']).isoformat()
			for bid_history in bid.get('bidHistories', []):
				bid_history['pickupDate'] = parse(bid_history['pickupDate']).strftime('%Y-%m-%d')
				bid_history['deliveryDate'] = parse(bid_history['deliveryDate']).strftime('%Y-%m-%d')
		bids = BidSchema().load(bids, many=True)
	return bids



from marshmallow import fields, EXCLUDE, post_load

from cc_py_commons.schemas.camel_case_schema import CamelCaseSchema
from cc_py_commons.transactions.equipment_clas_schema import EquipmentClassSchema
from cc_py_commons.transactions.transaction import Transaction

class TransactionSchema(CamelCaseSchema):
	class Meta:
		unknown = EXCLUDE

	client_id = fields.UUID()
	reference_number = fields.String()
	pickup_date = fields.Date()
	delivery_date = fields.Date()
	origin_city= fields.String()
	origin_state= fields.String()
	origin_postcode= fields.String()    
	destination_city= fields.String()
	destination_state= fields.String()
	destination_postcode= fields.String()    
	equipment= fields.String()
	equipment_class = fields.Nested(EquipmentClassSchema)
	all_in_cost = fields.Integer()
	contact_id = fields.UUID(allow_none=True)
	carrier_id = fields.UUID(allow_none=True)
	booking_date= fields.Date(allow_none=True)
	pickup_appointment_window= fields.Integer(allow_none=True)
	num_pickups= fields.Integer(allow_none=True)
	num_dropoffs= fields.Integer(allow_none=True)
	origin_country = fields.String(allow_none=True)
	origin_latitude = fields.Float(allow_none=True)
	origin_longitude = fields.Float(allow_none=True)
	destination_country = fields.String(allow_none=True)
	destination_latitude = fields.Float(allow_none=True)
	destination_longitude = fields.Float(allow_none=True)
	miles= fields.Integer(allow_none=True)
	line_haul_cost= fields.Integer(allow_none=True)
	fuel_portion= fields.Integer(allow_none=True)
	price_to_shipper= fields.Integer(allow_none=True)
	weight= fields.Integer(allow_none=True)
	commodity = fields.String(allow_none=True)
	comments = fields.String(allow_none=True)
	import_stats_id = fields.UUID(allow_none=True)
	#   import_stats = relationship("ImportStats", back_populates="loads", lazy='subquery')
	pickup_appointment_required = fields.Boolean(allow_none=True)
	delivery_appointment_required = fields.Boolean(allow_none=True)
	team_service = fields.Boolean(allow_none=True)
	hand_loading = fields.Boolean(allow_none=True)
	driver_assist_loading = fields.Boolean(allow_none=True)
	driver_assist_unloading = fields.Boolean(allow_none=True)
	drop_trailer = fields.Boolean(allow_none=True)
	hazmat = fields.Boolean(allow_none=True)
	lift_gate_required = fields.Boolean(allow_none=True)
	blanket_wrap = fields.Boolean(allow_none=True)
	white_glove = fields.Boolean(allow_none=True)
	home_delivery = fields.Boolean(allow_none=True)
	etrack_load_stabilization = fields.Boolean(allow_none=True)
	contract = fields.Boolean(allow_none=True)    
	straps_required = fields.Boolean(allow_none=True)
	tarps_required = fields.Boolean(allow_none=True)
	frozen = fields.Boolean(allow_none=True)
	origin_hub_weight = fields.Float(allow_none=True)
	destination_hub_weight = fields.Float(allow_none=True)
	calculated_miles= fields.Integer(allow_none=True)
	mileage_mismatch = fields.Boolean(allow_none=True)
	holiday = fields.Boolean(allow_none=True)
	accessorials_included= fields.Integer(allow_none=True)
	client_shipper_id = fields.String(allow_none=True)
	client_receiver_id = fields.String(allow_none=True)
	load_source_type = fields.String(allow_none=True)
	predicted_price= fields.Integer(allow_none=True)
	carrier_code = fields.String(allow_none=True)
	customer_code = fields.String(allow_none=True)
	customer_name = fields.String(allow_none=True)
	customer_id = fields.UUID(allow_none=True)
	account_exec_id = fields.UUID(allow_none=True)
	#   account_exec = relationship("AccountExec", lazy='subquery')
	carrier_rep_id = fields.UUID(allow_none=True)
	#   carrier_rep = relationship("CarrierRep", lazy='subquery')
	excluded = fields.Boolean(allow_none=True)
	lane_id = fields.UUID(allow_none=True)
	multi_stop = fields.Integer(default=1)
	margin = fields.Integer(allow_none=True)
	rate_type = fields.String(allow_none=True)
	provider = fields.String(allow_none=True)
	target_pay = fields.Integer(allow_none=True)
	max_buy = fields.Integer(allow_none=True)
	pieces = fields.Integer(allow_none=True)
	max_temperature = fields.Float(allow_none=True)
	min_temperature = fields.Float(allow_none=True)
	origin_pallets_required = fields.Boolean(allow_none=True)
	destination_pallets_required = fields.Boolean(allow_none=True)
	hazmat = fields.Boolean(allow_none=True)
	origin_location_id = fields.String(allow_none=None)
	origin_location_name = fields.String(allow_none=None)
	destination_location_id = fields.String(allow_none=None)
	destination_location_name = fields.String(allow_none=None)
	delivery_open_time = fields.Time(allow_none=True)
	delivery_close_time = fields.Time(allow_none=True)
	pickup_open_time = fields.Time(allow_none=True)
	pickup_close_time = fields.Time(allow_none=True)
	freight_hub_load_id = fields.UUID(allow_none=True)
	partial_load = fields.Boolean(allow_none=True)
	linear_feet = fields.Integer(allow_none=True)
	ltl = fields.Boolean(allow_none=True)
	revenue_code = fields.String(allow_none=True)
	request_id = fields.UUID(allow_none=True)
	mcleod_movement_id = fields.String(allow_none=True)
	user_id = fields.Integer(allow_none=True)
	
	@post_load
	def classify(self, data, **kwargs):
		print(f"post_load - {data}")
		return Transaction(**data)



from cc_py_commons.sns.sns_service import SnsService
from cc_py_commons.utils.logger_v2 import logger


class BidCounterFailureNotification:

    def __init__(self, app_config):
        self._app_config = app_config

    def send(self, bid_id, reason):
        subject = 'bid-counter-failure'
        message = '{' + f'"bidId" : "{bid_id}", ' \
                        f'"failureReason" : "{reason}", ' \
                        f'"subject" : "{subject}", ' \
                        f'"className":  "{self._app_config.BID_COUNTER_FAILURE_SNS_CLASS_NAME}"' + '}'
        logger.debug(f"sending BidCounterFailureNotification {message} to {self._app_config.BOOKING_AGENT_SNS_ARN}")

        try:
            sns_service = SnsService()
            sns_service.send(self._app_config.BOOKING_AGENT_SNS_ARN, subject, message)
        except Exception as e:
            logger.exception(f"failed to send message to {self._app_config.BOOKING_AGENT_SNS_ARN}: {message} - {e}", e)



import json
from uuid import uuid4
import boto3

class SqsService:
	def send(self, queue_url, message, delaySeconds=0, messageGroupId=None):
		'''
		Sends a message to the specified queue and returns the messageId. 
		The messageId can be used to lookup and delete the message in the queue.
		'''
		sqs = boto3.client('sqs')
		if messageGroupId:
			response = sqs.send_message(
				QueueUrl=queue_url,
				MessageBody=message,
				DelaySeconds=delaySeconds,
				MessageAttributes={},
				MessageGroupId=messageGroupId
			)
		else:
			response = sqs.send_message(
				QueueUrl=queue_url,
				MessageBody=message,
				DelaySeconds=delaySeconds,
				MessageAttributes={}
			)
		return response.get('MessageId')

def send_batches(queue_url, messages):
	'''
	Sends all messages to the specified queue in groups of 10 (the limit of send_batch)
	The messageId can be used to lookup and delete the message in the queue.
	Args:
		queue_url (str): The URL of the Amazon SQS queue
		messages (list): List of messages to be sent

	Returns:
		dict: Results containing 'successful' and 'failed' message lists
	'''
	results = {
		'successful': [],
		'failed': []
	}

	if not messages:
		return results

	sqs = boto3.client('sqs')
	entries = [
		{
			# each message must have a unique id
			'Id': str(uuid4()),
			'MessageBody': json.dumps(message),
		}
		for message in messages
	]

	# SQS can only process 10 messages per batch
	batch_size = 10

	# Split messages into batches of 10
	for i in range(0, len(entries), batch_size):
		batch = entries[i:i + batch_size]

		try:
			response = sqs.send_message_batch(
				QueueUrl=queue_url,
				Entries=batch
			)

			# Track successful and failed messages
			if 'Successful' in response:
				results['successful'].extend(response['Successful'])
			if 'Failed' in response:
				results['failed'].extend(response['Failed'])

		except Exception as e:
			# Mark all messages in failed batch
			failed_batch = [{
				'Id': entry['Id'],
				'Error': str(e),
				'Message': entry['MessageBody']
			} for entry in batch]
			results['failed'].extend(failed_batch)

	return results



import os
import math
from datetime import timedelta

from cc_py_commons.config.env import app_config


def execute(pickup_date, distance_in_miles=None):
	minimum_miles_per_day = app_config.MINIMUM_MILES_PER_DAY

	if distance_in_miles:
		if distance_in_miles <= minimum_miles_per_day:
			transit_days = math.floor(distance_in_miles / minimum_miles_per_day)
		else:
			transit_days = math.ceil(distance_in_miles / minimum_miles_per_day)
	else:
		transit_days = 0
	return pickup_date + timedelta(days=transit_days)



import os
import requests

from cc_py_commons.utils.logger_v2 import logger

MERCURY_URL = os.environ.get("MERCURY_URL")
MERCURY_TOKEN = os.environ.get("MERCURY_TOKEN")


def execute(params):
	url = f"{MERCURY_URL}/clients"
	token = f"Bearer {MERCURY_TOKEN}"
	headers = {
		"Authorization": token
	}
	response = requests.get(url, params=params, headers=headers)

	if response.status_code != 200:
		logger.warning(
			f"Listing clients failed for params: {params} - {response.status_code}:{response.text}")
		return None

	return response.json().get('content')




def is_event_from_lambda_warmer(event):
	result = False
	if 'Records' not in event:
		resources = event.get('resources', [])
		if len(resources) > 0 and 'LAMBDA_WARMER' in resources[0]:
			result = True
	return result



from marshmallow import fields, EXCLUDE, post_load
from cc_py_commons.schemas.camel_case_schema import CamelCaseSchema
from cc_py_commons.bids.bid import Bid
from cc_py_commons.bids.bid_history_schema import BidHistorySchema

class BidSchema(CamelCaseSchema):
  class Meta:
    unknown = EXCLUDE

  id = fields.UUID()
  quote_id = fields.UUID()
  carrier_id = fields.UUID()
  receipt_id = fields.String(allow_none=True, missing=None)
  amount = fields.Integer()
  estimated_days = fields.Integer(allow_none=True, missing=None)
  notes = fields.String(allow_none=True, missing=None)
  match_score = fields.Float(allow_none=True)
  status_id = fields.UUID()
  pickup_date = fields.Date()
  delivery_date = fields.Date()
  decline_reason = fields.String(allow_none=True, missing=None)
  bid_histories = fields.List(fields.Nested(BidHistorySchema), allow_none=True)
  origin_deadhead = fields.Float(allow_none=True)
  porus_truck_id = fields.UUID(allow_none=True, missing=None)
  truck_lane_id = fields.UUID(allow_none=True, missing=None)
  distance = fields.Float(allow_none=True, missing=None)
  origin_city = fields.String(missing=None)
  origin_state = fields.String(missing=None)
  origin_zip = fields.String(allow_none=True, missing=None)
  dest_city = fields.String(missing=None)
  dest_state = fields.String(missing=None)
  dest_zip = fields.String(allow_none=True, missing=None)
  invite_emailed_at = fields.DateTime(allow_none=True)
  
  @post_load
  def make_bid(self, data, **kwargs):
      return Bid(**data)
   


from datetime import date, datetime
from cc_py_commons.loads.load_schema import LoadSchema

def execute(freight_hub_json):
  freight_hub_json_copy = freight_hub_json.copy()

  pickup_date = freight_hub_json.get('pickupDate')
  if pickup_date:
    freight_hub_json_copy['pickupDate'] = date.fromtimestamp(int(pickup_date)/1000).isoformat()

  delivery_date = freight_hub_json.get('deliveryDate')
  if delivery_date:
    freight_hub_json_copy['deliveryDate'] = date.fromtimestamp(int(delivery_date)/1000).isoformat()

  available_time = freight_hub_json.get('availableTime')
  if available_time:
    freight_hub_json_copy['availableTime'] = datetime.fromtimestamp(int(available_time)/1000).isoformat()

  return LoadSchema().load(freight_hub_json_copy)



import os
import requests

from cc_py_commons.utils.logger_v2 import logger

FREIGHT_HUB_URL = os.environ.get('FREIGHT_HUB_URL')
FREIGHT_HUB_TOKEN = os.environ.get("FREIGHT_HUB_TOKEN")

def execute(load):
  url = f"{FREIGHT_HUB_URL}/freight"
  token = f"Bearer {FREIGHT_HUB_TOKEN}"
  headers = {
    "Authorization": token
  }
  
  logger.debug(f"post_freight_hub_load: Posting to freight-hub: {url}, {headers}, {load}")
  return requests.post(url, json=load, headers=headers)



from marshmallow import fields, EXCLUDE, post_load

from cc_py_commons.schemas.camel_case_schema import CamelCaseSchema
from cc_py_commons.transactions.equipment_class import EquipmentClass
from cc_py_commons.transactions.equipment_mapping import EquipmentMapping

class EquipmentClassSchema(CamelCaseSchema):
    class Meta:
        unknown = EXCLUDE

    equipment_class = fields.Nested(EquipmentClass)
    mapped_text = fields.String()

    @post_load
    def classify(self, data, **kwargs):
        return EquipmentClass(**data)        


import psycopg2

from cc_py_commons.config.env import app_config

from cc_py_commons.utils.logger_v2 import logger

db_host = app_config.DB_HOST
db_user = app_config.DB_USERNAME
db_password = app_config.DB_PASSWORD
db_name = app_config.DB_NAME

def get():
	try:
		conn_string = f"host={db_host} user={db_user} password={db_password} dbname={db_name}"
		return psycopg2.connect(conn_string)
	except Exception as e:
		logger.error('db.get_connection: Failed with error: {e}', exc_info=True)
	
	return None

def close(connection):
	if connection:
			connection.close()




class LoadFieldConstants:

  ACCOUNT_ID = 'accountId'
  REFERENCE_NUMBER = 'referenceNumber'
  BOOKING_DATE = 'bookingDate'
  PICKUP_DATE = 'pickupDate'
  DELIVERY_DATE = 'deliveryDate'
  ORIGIN_CITY = 'originCity'
  ORIGIN_STATE = 'originState'
  DESTINATION_CITY = 'destinationCity'
  DESTINATION_STATE = 'destinationState'
  EQUIPMENT = 'equipment'
  CARRIER_ID = 'carrierId'
  ALL_IN_COST = 'allInCost'
  PRICE_TO_SHIPPER = 'priceToShipper'
  MILES = 'miles'
  CLIENT_ID = 'clientId'
  IMPORT_STATS_ID = 'importStatsId'
  CARRIER_CODE = 'carrierCode'
  ACCOUNT_EXEC_CODE = 'accountExecCode'
  ACCOUNT_EXEC_FIRST_NAME = 'accountExecFirstName'
  ACCOUNT_EXEC_LAST_NAME = 'accountExecLastName'
  CUSTOMER_CODE = 'customerCode'
  CUSTOMER_NAME = 'customerName'
  CARRIER_NAME = 'carrierName'
  CARRIER_REP_CODE = 'carrierRepCode'
  CARRIER_REP_FIRST_NAME = 'carrierRepFirstName'
  CARRIER_REP_LAST_NAME = 'carrierRepLastName'
  MARGIN = 'margin'
  RATE_TYPE = 'rateType'
  PROVIDER = 'provider'
  LINE_HAUL_COST = 'lineHaulCost'
  FUEL_PORTION = 'fuelPortion'
  ACCESSORIALS_INCLUDED = 'accessorialsIncluded'
  COMMODITY = 'commodity'
  WEIGHT = 'weight'
  ORIGIN_POSTCODE = 'originPostcode'
  DESTINATION_POSTCODE = 'destinationPostcode'
  REQUEST_ID = 'requestId'

class FileType:

  SINGLE_ROW = 'single_row'
  DUAL_ROW = 'dual_row'
  ALJEX_FILE = 'aljex_file'


import os
import requests

from cc_py_commons.utils.logger_v2 import logger

MERCURY_URL = os.environ.get("MERCURY_URL")
MERCURY_TOKEN = os.environ.get("MERCURY_TOKEN")


'''
	Either returns lane price data or None. 
	This does not provide error details to the consumer in case the lookup fails.
'''
def execute(params):
	url = f"{MERCURY_URL}/lanePricing"
	token = f"Bearer {MERCURY_TOKEN}"
	headers = {
		"Authorization": token
	}
	response = requests.get(url, params=params, headers=headers)

	if response.status_code != 200:
		logger.warning(
			f"Lane Pricing lookup failed for params: {params} - {response.status_code}:{response.text}")
		return None

	return response.json()

'''
	Returns a dict with data and error key. 
	data refers to the lane price data if the price was generated.
	error contains the stringified status code and text of Mercury response in case the lookup failed.
'''
def execute_v2(params):
	result = {
		'data': None,
		'error': None
	}
	url = f"{MERCURY_URL}/lanePricing"
	token = f"Bearer {MERCURY_TOKEN}"
	headers = {
		"Authorization": token
	}
	response = requests.get(url, params=params, headers=headers)

	if response.status_code != 200:
		error = f'{response.status_code}:{response.text}'
		result['error'] = error
		logger.warning(f"Lane Pricing lookup failed for params: {params} - {error}")
	else:
		result['data'] = response.json()
	return result



import requests

from cc_py_commons.utils import json_logger
from .constants import C4_API_AUTH_TOKEN, C4_API_URL
from . import get_account


def by_user_id(user_id):
	url = f"{C4_API_URL}/user/{user_id}/accountSettings"
	token = f"Bearer {C4_API_AUTH_TOKEN}"
	headers = {
		"Authorization": token
	}
	json_logger.debug(None, 'Getting account settings', url=url, headers=headers)
	response = requests.get(url, headers=headers)
	if response.status_code != 200:
		json_logger.warning(None, 'Request to get account settings failed', status_code=response.status_code)
		return None
	return response.json().get('data')


def by_account_id(account_id):
	account = get_account.execute(account_id)
	if account:
		return account.get('accountSettings')
	else:
		return None



import os
import pandas as pd

from cc_py_commons.utils.logger_v2 import logger


def load_data(filename):
  path = os.path.join(os.getcwd(), 'test_data', 'raw', filename)
  logger.debug(f"load_data: loading from {path}")

  if (".csv" in filename.lower()):
    data = pd.read_csv(path)
  else:
    data = pd.read_excel(path)

  return data

def load_data_as_string(filename, skip_rows=0):
  path = os.path.join(os.getcwd(), 'test_data', 'raw', filename)
  logger.debug(f"load_data_as_string: loading from {path}")

  if (".csv" in filename.lower()):
    data = pd.read_csv(path)
  else:
    data = pd.read_excel(path, skiprows=skip_rows, dtype=str)

  return data

def load_data_as_string_without_headers(filename, skip_rows=0):
  path = os.path.join(os.getcwd(), 'test_data', 'raw', filename)
  logger.debug(f"load_data_as_string: loading from {path}")

  if (".csv" in filename.lower()):
    data = pd.read_csv(path, skiprows=skip_rows, header=None)
  else:
    data = pd.read_excel(path, skiprows=skip_rows, dtype=str, header=None)

  return data



import requests
import os
from dateutil.parser import parse

from cc_py_commons.utils.logger_v2 import logger
from cc_py_commons.quotes.quote_schema import QuoteSchema

BOOKING_AGENT_URL = os.environ.get('BOOKING_AGENT_URL')
BOOKING_AGENT_TOKEN = os.environ.get("BOOKING_AGENT_TOKEN")


def execute(quote_id):
  url = f"{BOOKING_AGENT_URL}/quote/{quote_id}"
  token = f"Bearer {BOOKING_AGENT_TOKEN}"
  headers = {
    "Authorization": token
  }

  logger.debug(f"Requesting quote {quote_id} from booking-agent: {url}, {headers}")
  response = requests.get(url, headers=headers)
  quote = None

  if response.status_code == 200:
    json = response.json()
    logger.debug(f"quote response: {json}")
    json['pickupDate'] = parse(json['pickupDate']).strftime('%Y-%m-%d')
    json['deliveryDate'] = parse(json['deliveryDate']).strftime('%Y-%m-%d')
    quote = QuoteSchema().load(json)

  logger.debug(f"quote response: {quote}")
  return quote



import json, traceback
from dataclasses import fields
from cc_py_commons.transactions.transaction import Transaction
from cc_py_commons.utils.get_delivery_date import execute as get_delivery_date
from cc_py_commons.utils.logger_v2 import logger

def from_load(load):
	"""Takes a freight-hub load and maps it to a pricing load (Transaction)"""
	trans_data = {}

	for field in fields(Transaction):
		value = getattr(load, field.name, None)
		
		if value:
			trans_data[field.name] = value

	if not trans_data.get('delivery_date') and load.pickup_date:
		trans_data['delivery_date'] = get_delivery_date(pickup_date=load.pickup_date, distance_in_miles=load.distance)

	trans_data['equipment'] = load.equipment_description
	trans_data['all_in_cost'] = load.rate # all in cost may change at booking
	trans_data['carrier_id'] = load.carrier_id
	trans_data['target_pay'] = load.rate # target_pay is what the broker listed the load for
	trans_data['multi_stop'] = load.stops_count
	trans_data['team_service'] = load.team_service_required
	trans_data['partial_load'] = load.partial_load
	trans_data['ltl'] = load.ltl
	
	if isinstance(load.origin, dict):
		trans_data['origin_city'] = load.origin['city']
		trans_data['origin_state'] = load.origin['state']
		trans_data['origin_postcode'] = load.origin['postcode']
	else:
		trans_data['origin_city'] = load.origin.city
		trans_data['origin_state'] = load.origin.state
		trans_data['origin_postcode'] = load.origin.postcode

	if isinstance(load.destination, dict):
		trans_data['destination_city'] = load.destination['city']
		trans_data['destination_state'] = load.destination['state']
		trans_data['destination_postcode'] = load.destination['postcode']
	else:
		trans_data['destination_city'] = load.destination.city
		trans_data['destination_state'] = load.destination.state
		trans_data['destination_postcode'] = load.destination.postcode

	del trans_data['customer_id']
	trans_data['client_id'] = None
	trans_data['equipment_class'] = None
	trans_data['origin_pallets_required'] = load.origin_pallets_required
	trans_data['destination_pallets_required'] = load.destination_pallets_required
	trans_data['hazmat'] = load.hazmat
	trans_data['origin_location_id'] = load.origin_location_id
	trans_data['origin_location_name'] = load.origin_location_name
	trans_data['destination_location_id'] = load.destination_location_id
	trans_data['destination_location_name'] = load.destination_location_name
	trans_data['delivery_open_time'] = load.delivery_open_time
	trans_data['delivery_close_time'] = load.delivery_close_time
	trans_data['pickup_open_time'] = load.pickup_open_time
	trans_data['pickup_close_time'] = load.pickup_close_time
	trans_data['freight_hub_load_id'] = load.id
	trans_data['revenue_code'] = load.revenue_code
	trans_data['user_id'] = load.customer_id
	if load.special_instructions:
		trans_data['comments'] = load.special_instructions
	if load.linear_feet:
		trans_data['linear_feet'] = load.linear_feet

	try:
		transaction = Transaction(**trans_data)
	except Exception as e:
		logger.warning(json.dumps({
			'message': 'Failed to map Transaction schema object',
			'error': str(e),
			'stacktrace': traceback.format_exc()
		}))
		return None
	
	return transaction



import pytz
from dateutil.parser import parse

##
# Date, Time, and DateTime handling functions.
# The dateutil library is used to have robust string parsing.
##


def parse_date_str_to_datetime(date_str):
  return parse(date_str)


def to_utc(input_datetime):
  """
  Take a datetime of any timezone and returns a datetime in UTC
  """
  return input_datetime.astimezone(pytz.UTC)


def is_same_date(date1, date2):
  return date1.strftime('%Y-%m-%d') == date2.strftime('%Y-%m-%d')


def format_datetime(datetime_object, format = None):
  """
  Takes a datetime object and returns string of the specified format
  """
  DEFAULT_FORMAT = '%Y-%m-%d %H:%M:%S'
  format = format or DEFAULT_FORMAT
  if datetime_object:
    return datetime_object.strftime(format)




def kg_to_lbs(weight_in_kg):
	weight_in_lbs = None
	if weight_in_kg:
		weight_in_lbs = round(weight_in_kg * 2.2046, 2)
	return weight_in_lbs

def tons_to_lbs(weight_in_tons):
	weight_in_lbs = None
	if weight_in_tons:
		weight_in_lbs = round(weight_in_tons * 2000, 2)
	return weight_in_lbs

def inch_to_ft(length_in_inch):
	length_in_ft = None
	if length_in_inch:
		length_in_ft = round(length_in_inch * 0.0833, 2)
	return length_in_ft

def centimeter_to_ft(length_in_cm):
	length_in_ft = None
	if length_in_cm:
		length_in_ft = round(length_in_cm * 0.032808, 2)
	return length_in_ft

def meter_to_ft(length_in_m):
	length_in_ft = None
	if length_in_m:
		length_in_ft = round(length_in_m * 3.28084, 2)
	return length_in_ft



import json

from cc_py_commons.sns.sns_service import SnsService
from cc_py_commons.utils.logger_v2 import logger

class BookingAssistantFlowNotification:

  def __init__(self, app_config):
    self._app_config = app_config

  def send(self, user_id, load, params):
    message = {
      'userId': user_id,
      'loadDTO': load,
      'payload': params,
      'subject': f'{self._app_config.BOOKING_ASSISTANT_SNS_SUBJECT}',
      'className': f'{self._app_config.BOOKING_ASSISTANT_SNS_CLASS_NAME}'
    }
    logger.debug(f"sending messsage {message} to {self._app_config.BOOKING_ASSISTANT_SNS_TOPIC_ARN}")

    try:
      sns_service = SnsService()
      sns_service.send(self._app_config.BOOKING_ASSISTANT_SNS_TOPIC_ARN,
        self._app_config.BOOKING_ASSISTANT_SNS_SUBJECT, json.dumps(message))
    except Exception as e:
      logger.error(f"failed to send BookingAssistantFlowNotification message: {e}")
      logger.error(f"BookingAssistantFlowNotification message: {message}")



import uuid

from dataclasses import dataclass, field

from cc_py_commons.transactions.equipment_class import EquipmentClass

@dataclass
class EquipmentMapping:

  equipment_class: EquipmentClass
  mapped_text: str
  # generated by server and returned
  id: uuid.UUID = field(default=None)


import requests

from cc_py_commons.utils.logger_v2 import logger
from .constants import C4_API_AUTH_TOKEN, C4_API_URL


def execute(account_id, integration_type, request_id=None):
	url = f"{C4_API_URL}/integrations"
	token = f"Bearer {C4_API_AUTH_TOKEN}"
	headers = {
		"Authorization": token
	}
	params = {
		'accountId': account_id,
		'integrationType': integration_type,
		'requestId': request_id
	}
	logger.debug(f"Getting integration for account: {account_id} using URL: {url}, Params: {params}")
	response = requests.get(url, headers=headers, params=params)

	if response.status_code != 200:
		logger.warning(f"Request to get account integration failed with status code: "
									 f"{response.status_code}")
		return None

	json = response.json()

	if json['code'] != 200:
		logger.warning(f"notifications: Unable to find {integration_type} integration for account Id: {account_id}")
		return False

	return json



from cc_py_commons.config.c4_account_settings import C4AccountSettings


def execute(lane_price_search, account_settings, user_id):
	return {
		'trailerTypes': [lane_price_search.get('equipment')],
		'originCity': lane_price_search.get('originCity'),
		'originState': lane_price_search.get('originState'),
		'originZip': lane_price_search.get('originPostcode'),
		'originLat': lane_price_search.get('originLatitude'),
		'originLng': lane_price_search.get('originLongitude'),
		'destCity': lane_price_search.get('destinationCity'),
		'destState': lane_price_search.get('destinationState'),
		'destZip': lane_price_search.get('destinationPostcode'),
		'destLat': lane_price_search.get('destinationLatitude'),
		'destLng': lane_price_search.get('destinationLongitude'),
		'originDeadhead': account_settings.get(C4AccountSettings.SEARCH_TRUCK_ORIGIN_DEADHEAD_LIMIT),
		'destinationDeadhead': account_settings.get(C4AccountSettings.SEARCH_TRUCK_DESTINATION_DEADHEAD_LIMIT),
		'resultCount': account_settings.get(C4AccountSettings.SEARCH_TRUCK_RESULTS_COUNT),
		'pickupDateStart': lane_price_search.get('pickupDate'),
		'pickupDateEnd': lane_price_search.get('deliveryDate'),
		'accountId': lane_price_search.get('accountId'),
		'userId': user_id,
		'userType': 'USER',
		'inNetworkCarriers': True
	}



import boto3

class SnsService:
    def send(self, topic_arn, subject, message):
        sns = boto3.client('sns')
        sns.publish(
            TopicArn=topic_arn,
            Subject=subject,
            Message=message,
            MessageAttributes={}
        )



import json
import traceback

from cc_py_commons.config.env import app_config
from cc_py_commons.sqs.sqs_service import SqsService
from cc_py_commons.utils.logger_v2 import logger

class BookingAssistantQuoteCancelNotification:

	def send(self, load_id):
		'''
		Sends and event to the Booking Assistant Flow SQS queue specified in the config.
		Returns the messageId of the enqueued message for later retrieval.
		'''
		event = {
			'loadId': str(load_id),
			'subject': app_config.BOOKING_ASSISTANT_QUOTE_CANCEL_SUBJECT,
			'className': app_config.BOOKING_ASSISTANT_QUOTE_CANCEL_CLASS_NAME
		}
		logger.debug(f"sending event {event} to {app_config.BOOKING_ASSISTANT_FLOW_QUEUE}")

		try:
			sqs_service = SqsService()
			return sqs_service.send(app_config.BOOKING_ASSISTANT_FLOW_QUEUE, json.dumps(event), app_config.BOOKING_ASSISTANT_DELAY_SECONDS)
		except Exception as e:
			logger.error(json.dumps({
				'message': 'Failed to send class BookingAssistantQuoteCancelNotification',
				'error': str(e),
				'sta_cktrace': traceback.format_exc(),
				'event': event
			}))



import math


def convert_to_cents(price_in_dollars):
  return (price_in_dollars * 100) if price_in_dollars is not None else None

def convert_to_cents_and_round(price_in_dollars, precision=None):
  price_in_cents = convert_to_cents(price_in_dollars)
  if price_in_cents:
    if precision:
      return round(price_in_cents, precision)
    else:
      return round(price_in_cents)
  return None

def convert_to_dollars(price_in_cents):
  return (price_in_cents / 100) if price_in_cents is not None else None

def convert_to_dollars_and_round(price_in_cents, precision=None):
  price_in_dollars = convert_to_dollars(price_in_cents)
  if price_in_dollars:
    if precision:
      return round(price_in_dollars, precision)
    else:
      return round(price_in_dollars)
  return None

def extract_amount_in_cents(value):
  parsed_value = value

  if type(value) is str:
    if len(value) > 0:
      clean_value = value.replace(',', '').replace('$', '')
      if '.' in clean_value:
        clean_value = clean_value.split('.')[0]
      parsed_value = float(clean_value)
  else:
    parsed_value = float(value)

  if parsed_value and not math.isnan(parsed_value):
    return int(parsed_value * 100) #convert to cents
  else:
    return None
